import { CallHandler, ExecutionContext, Injectable, NestInterceptor } from '@nestjs/common';
import { map } from 'rxjs/operators';
import { Observable } from 'rxjs';

import { EntityType } from '../types/entity.type';
import { Response } from '../interfaces/response.interface';

@Injectable()
export class EntitySerializerInterceptor<T> implements NestInterceptor<T, Response<T>> {
  constructor(private entity: EntityType<T>) { }

  public intercept(context: ExecutionContext, next: CallHandler): Observable<Response<T>> {
    const relations = Reflect.getMetadata('SERIALIZER_RELATION', this.entity) || {};
    return next.handle().pipe(map(data => ({ data, relations })));
  }
}